/*
   program : sl2recon

   $Source: /home/yee/C/RCS/sl2recon.c,v $
   $Author: yee $
   $Date: 1999/02/10 16:37:41 $
   $Revision: 1.2 $

   This generates the Shepp-Logan head phantom.

   Compile Options : none

   Parameters : see the documentation for ParseArg()

   Assumptions : none known

   Bugs : none known

   To Do :

   $Log: sl2recon.c,v $
   Revision 1.2  1999/02/10 16:37:41  yee
   *** empty log message ***

   Revision 1.1  1999/02/05 03:54:16  yee
   Initial revision

   Copyright 1999 by Conway Yee.  All rights reserved.
 */

char recon[] = "$Id: sl2recon.c,v 1.2 1999/02/10 16:37:41 yee Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "style.h"
#include "array.h"
#include "mutil.h"
#include "util.h"

#include "backproject.h"
#include "ImageHeader.h"
#include "SheppLogan.h"

PUBLIC char *optarg;               /* used by getopt() */
PUBLIC int optind=0;
PUBLIC int opterr=0;
PUBLIC int optopt=0;

BOOLEAN verbose=FALSE;

PRIVATE ImageHeader *ParseArg(ImageHeader *,int,char **,FILE **);
PRIVATE void SetSheppLogan(ImageHeader *h);
PRIVATE BOOLEAN InEllipse(double x,double y,int el);

/*****************************************************************************/
/*****************************************************************************/
 
/* this the main module driver for filter backprojection. */

PUBLIC int main(int argc,char **argv)

{
  ImageHeader *h;                 /* SheppLogan image headerp*/
  FILE *outptr=stdout;            /* output file pointer */

  SetupSL();   /* set up Shepp-Logan phantom */

  AllocateImageHeader(h,NDIM);
  ImageHeaderEditPersonalData(h->name,"Shepp-Logan Head Phantom");
  ImageHeaderEditPersonalData(h->loc,"IEEE Transactions Nuclear Science");
  ImageHeaderEditPersonalData(h->mr,"NS-21, 6/74, pgs 21-43");

  h=ParseArg(h,argc,argv,&outptr);
  h->fptr=outptr;

  h->data.f=(RECON_FLOAT **)
                    mdmalloc(NDIM,sizeof(RECON_FLOAT),ALLOC_DISCRETE,h->lim);
  if(h->data.f<0) {
    perror("sl2recon : mdmalloc failure on reconstructed image array ");
    return(errno);
  }

  SetSheppLogan(h);                                    /* reconstruct image */

  if(!ioImageHeader(h,fwrite,outptr)) {            /* output image */
    errno=HEADER_ERROR;
    perror("sl2recon : unable to write output header ");
    return(errno);
  }
  mdio(outptr,2,sizeof(RECON_FLOAT),sizeof(RECON_FLOAT),h->data.f,fwrite,
       h->lim);

  FreeImageHeader(h);                                   /* clean up */
  fclose(outptr);

  return(errno);
}

/*****************************************************************************/

/* this function determines the image of the Shepp-Logan head phantom */

PRIVATE void SetSheppLogan
(
 ImageHeader *h    /* image header */
)
{
  int i;               /* X counter */

  for(i=h->lim[LOX];i<=h->lim[HIX];i++) { /* loop on i */
    int j;             /* Y counter */
    double x=i*h->tau[X];                                 /* X position */
    for(j=h->lim[LOY];j<=h->lim[HIY];j++) { /* loop on j */
      double y=j*h->tau[Y];              /* Y position */
      int k;                                 /* ellipse counter */
      for(h->data.f[i][j]=0.0,k=0;k<MAX_SL;k++)
        if(InEllipse(x,y,k)) h->data.f[i][j] += SL[k][INDEX];
    } /* loop on j */
  } /* loop on i */
}

/*****************************************************************************/

/* is position (x,y) within ellipse k? */

PRIVATE BOOLEAN InEllipse
(
 double x,    /* x position */
 double y,    /* y position */
 int el       /* el'th ellipse of Shepp-Logan phantom */
)
{
  x -= X1;   /* correct for spatial offset */
  y -= Y1;

  { /* correct for rotation */
    double xt=x*cos(ALPHA)+y*sin(ALPHA);       /* temporary x variable */
    double yt=y*cos(ALPHA)-x*sin(ALPHA);       /* temporary y variable */

    x=xt;
    y=yt;
  } /* correct for rotation */

  return((SQR(x/A)+SQR(y/B))<1.0?TRUE:FALSE);
}

/*****************************************************************************/

/* This function processes the command line arguments of sl2recon.
   See the declaration of *errormsg[] for legal options. */

PRIVATE ImageHeader *ParseArg
(
 ImageHeader *h,             /* image header */
 int argc,                   /* argument count */
 char **argv,                /* arguments */
 FILE **outptr               /* output file pointer */
)
{ /* ParseArg() */
  char opt;                                                        /* option */
  char *outfname=NULL;                            /* filename of output file */
  BOOLEAN error=FALSE;                        /* tests if error in arguments */
  double *range=calloc(1,sizeof(double)*NDIM);              /* field of view */

  char options[]="o:X:Y:x:y:v";
  char *errormsg[]={
    "sl2recon [-o <name>] [-X <float>] [-Y <float>] [-x <int>] [-y <int>]",
    "   -o <output file name>",
    "   -X <range in X>",
    "   -Y <range in Y>",
    "   -x <number of pixels in X>",
    "   -y <number of pixels in Y>",
    "   -v verbose mode",
    NULL
  };

  do {
    opt = getopt(argc,argv,options);
    switch(opt) {
    case 'o':                                             /* output filename */
      *outptr=fopen(optarg,"w+");
      if(*outptr == NULL) {
        perror("sl2recon : unable to open output file");
        error=TRUE;
      } else {
        outfname=malloc(strlen(optarg)+1);
        strcpy(outfname,optarg);
      }
      break;
    case 'X':                                             /* range in X */
      range[X] = atof(optarg);
      if ((errno == ERANGE)||(range[X]<0)) {
        perror("sl2recon : illegal distance between samples ");
        error=TRUE;
      }
      break;
    case 'Y':                                             /* range in Y */
      range[Y] = atof(optarg);
      if ((errno == ERANGE)||(range[Y]<0)) {
        perror("sl2recon : illegal distance between samples ");
        error=TRUE;
      }
      break;
    case 'x':                                       /* number of pixels in X */
      {
        int num;               /* number read as option */
        h->dim[X] = num = atoi(optarg);
        h->lim[LOX] = num/2 - num+1;
        h->lim[HIX] = num/2;
        if ((errno == ERANGE)||(h->ndim<0)) {
          perror("sl2recon : illegal # of pixels in X ");
          error=TRUE;
        }
      }
      break;
    case 'y':                                       /* number of pixels in X */
      {
        int num;               /* number read as option */
        h->dim[Y] = num = atoi(optarg);
        h->lim[LOY] = num/2 - num+1;
        h->lim[HIY] = num/2;
        if ((errno == ERANGE)||(h->lim[HIY]<0)) {
          perror("sl2recon : illegal # of pixels in Y ");
          error=TRUE;
        }
      }
      break;
    case 'v':                                                /* verbose mode */
      verbose=TRUE;
      break;
    case ':':                                           /* missing parameter */
      perror("sl2recon : missing parameter in option");
      error=TRUE;
      break;
    case '?':                                              /* illegal option */
      perror("sl2recon : illegal option");
      error=TRUE;
      break;
    case -1:                                           /* all options parsed */
      if((h->dim[Y]<=0)||(h->dim[X]<=0)) {   /* number of pixels */
        h->lim[HIX]=h->lim[HIY]=MAX(h->lim[HIX],h->lim[HIY]);
        h->lim[LOX]=h->lim[LOY]=MIN(h->lim[LOX],h->lim[LOY]);
        h->dim[X]=h->dim[Y]=MAX(h->dim[X],h->dim[Y]);
        if(h->lim[HIX]>0)
          fprintf(stderr,"sl2recon : setting square image : %1d\n",h->lim[HIX]);
        else {
          fprintf(stderr,"sl2recon : pixel number not set\n");
          error=TRUE;
        }
      }  /* number of pixels */

      { /* set pixel size */
        int i; /* counter */
        for(i=0;i<NDIM;i++) h->tau[i] = range[i]/h->dim[i];
      } /* set pixel size */

      if((h->tau[X]<=0.0)||(h->tau[Y]<=0.0)) {  /* pixel size */
        h->tau[X]=h->tau[Y]=MAX(h->tau[X],h->tau[Y]);
        if(h->tau[X]>0)
          fprintf(stderr,"sl2recon : setting square pixel : %g\n",h->tau[X]);
        else {
          fprintf(stderr,"sl2recon : pixel size not set\n");
          error=TRUE;
        }
      }  /* pixel size */
       
      if(error) {
        USAGE(errormsg)
      } else if(verbose) {
        fprintf(stderr,"sl2recon : output filename : %s\n",outfname);
        fprintf(stderr,"sl2recon : %f distance between pixels in X\n",h->tau[X]);
        fprintf(stderr,"sl2recon : %f distance between pixels in Y\n",h->tau[Y]);
        fprintf(stderr,"sl2recon : %d pixels in X\n",h->dim[X]);
        fprintf(stderr,"sl2recon : %d pixels in Y\n",h->dim[Y]);
      }

      free(outfname);

      break;
    default :
      errno = ILLEGAL_ARGUMENTS;
      error=TRUE;
    }
  } while(opt != -1);

  free(range);
  return(h);

} /* ParseArg() */
